# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Public TensorFlow type definitions.

For details, see
https://github.com/tensorflow/community/blob/master/rfcs/20200211-tf-types.md.

"""

import sys as _sys

from . import experimental